﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Threading;

namespace APIcheck
{
    public static class EveData
    {
        public static EveData.JSONData Data;
        public static Thread DataThread;

        public class JSONData
        {

            public bool FetchComplete = false;
            private Dictionary<string, SolarSystem> _SolarSystems;
            private Dictionary<string, Station> _Stations;
            private Dictionary<string, Region> _Regions;
            private Dictionary<string, Type> _Types;
            private Dictionary<string, Agent> _Agents;
            private Dictionary<string, EVE.Net.SkillTree.Group> _SkillGroups;
            private Dictionary<string, EVE.Net.SkillTree.Skill> _Skills;

            #region Accessors
            public Dictionary<string, SolarSystem> SolarSystems
            {
                get
                {
                    this.Complete();
                    return this._SolarSystems;
                }
            }

            public Dictionary<string, Region> Regions
            {
                get
                {
                    this.Complete();
                    return this._Regions;
                }
            }

            public Dictionary<string, Type> Types
            {
                get
                {
                    this.Complete();
                    return this._Types;
                }
            }

            public Dictionary<string, Station> Stations
            {
                get
                {
                    this.Complete();
                    return this._Stations;
                }
            }

            public Dictionary<string, Agent> Agents
            {
                get 
                {
                    this.Complete();
                    return this._Agents;
                }
            }

            public Dictionary<string, EVE.Net.SkillTree.Group> SkillGroups
            {
                get
                {
                    this.Complete();
                    return this._SkillGroups;
                }
            }

            public Dictionary<string, EVE.Net.SkillTree.Skill> Skills
            {
                get
                {
                    this.Complete();
                    return this._Skills;
                }
            }
            #endregion Accessors

            public JSONData()
            {
                this._SolarSystems = new Dictionary<string, SolarSystem>();
                this._Stations = new Dictionary<string, Station>();
                this._Regions = new Dictionary<string, Region>();
                this._Types = new Dictionary<string, Type>();
                this._Agents = new Dictionary<string, Agent>();
                this._SkillGroups = new Dictionary<string, EVE.Net.SkillTree.Group>();
                this._Skills = new Dictionary<string, EVE.Net.SkillTree.Skill>();
            }

            /// <summary>
            /// Makes sure the thread has finished preloading the data.
            /// </summary>
            public void Complete()
            {
                if (!this.FetchComplete)
                {
                    EveData.DataThread.Join();
                }
            }

            /// <summary>
            /// Fetches static Eve data from JSON.
            /// </summary>
            /// <param name="data">An EveData object to be filled.</param>
            public static void Fetch(object data)
            {
                if (!(data is EveData.JSONData))
                {
                    return;
                }
                EveData.JSONData obj = (data as EveData.JSONData);

                if (!Directory.Exists("data"))
                {
                    return;
                }

                if (!File.Exists("data/SolarSystem.js") || !File.Exists("data/Region.js") || !File.Exists("data/Type.js")
                    || !File.Exists("data/Agent.js") || !File.Exists("data/Station.js") || !File.Exists("data/SkillRequirements.js"))
                {
                    return;
                }

                try
                {
                    // the JSON reader returns an arraylist of solar systems in a hashtable
                    ArrayList ssdata = (ArrayList)JSON.JsonDecode(File.ReadAllText("data/SolarSystem.js"));
                    foreach (Hashtable h in ssdata)
                    {
                        SolarSystem newss = new SolarSystem(h);
                        obj._SolarSystems.Add(newss.ID, newss);
                    }

                    // regions
                    ArrayList regiondata = (ArrayList)JSON.JsonDecode(File.ReadAllText("data/Region.js"));
                    foreach (Hashtable h in regiondata)
                    {
                        Region newregion = new Region(h);
                        obj._Regions.Add(newregion.ID, newregion);
                    }

                    // types
                    ArrayList typedata = (ArrayList)JSON.JsonDecode(File.ReadAllText("data/Type.js"));
                    foreach (Hashtable h in typedata)
                    {
                        Type newtype = new Type(h);
                        obj._Types.Add(newtype.ID, newtype);
                    }

                    // stations
                    ArrayList stationdata = (ArrayList)JSON.JsonDecode(File.ReadAllText("data/Station.js"));
                    foreach (Hashtable h in stationdata)
                    {
                        Station newstation = new Station(h);
                        obj._Stations.Add(newstation.ID, newstation);
                    }

                    // agents
                    ArrayList agentdata = (ArrayList)JSON.JsonDecode(File.ReadAllText("data/Agent.js"));
                    foreach (Hashtable h in agentdata)
                    {
                        Agent newagent = new Agent(h);
                        obj._Agents.Add(newagent.ID, newagent);
                    }

                    // skill requirements
                    Hashtable skilldata = (Hashtable)JSON.JsonDecode(File.ReadAllText("data/SkillRequirements.js"));
                    foreach (DictionaryEntry pair in skilldata)
                    {
                        // key should be a string containing the typeid, value should be a hashtable
                        if (!(pair.Key is string) || !(pair.Value is Hashtable))
                        {
                            continue;
                        }
                        string key = (string)pair.Key;
                        Hashtable value = (Hashtable)pair.Value;

                        // find the type to wedge in
                        if (!obj._Types.ContainsKey(key))
                        {
                            continue;
                        }
                        Type t = obj._Types[key];

                        // loop through the skills, there can only be 3 skills
                        for (int i = 0; i <= 2; i++)
                        {
                            if (value.ContainsKey(SkillRequirement.AttribIDByOrder[i].ToString()) && value.ContainsKey(SkillRequirement.LevelIDByOrder[i].ToString()))
                            {
                                string skillid = value[SkillRequirement.AttribIDByOrder[i].ToString()].ToString();
                                int level = Convert.ToInt32((double)value[SkillRequirement.LevelIDByOrder[i].ToString()]);

                                if (!obj._Types.ContainsKey(skillid))
                                {
                                    continue;
                                }

                                SkillRequirement sr = new SkillRequirement(obj._Types[skillid], level, (SkillRequirement.RequirementOrder)i);
                                t.SkillRequirements.Add(sr);
                            }
                        }
                    }

                    // not json, but still evewide data; skills
                    EVE.Net.SkillTree skills = new EVE.Net.SkillTree();
                    skills.Query();

                    foreach (EVE.Net.SkillTree.Group g in skills.skillGroups)
                    {
                        // for some reason some groups are split in the API
                        // if a group already exists, add the skills to it
                        if (obj._SkillGroups.ContainsKey(g.groupID.ToString()))
                        {
                            obj._SkillGroups[g.groupID.ToString()].skills.AddRange(g.skills);
                        }
                        else
                        {
                            obj._SkillGroups.Add(g.groupID.ToString(), g);
                        }

                        foreach (EVE.Net.SkillTree.Skill s in g.skills)
                        {
                            obj._Skills.Add(s.typeID.ToString(), s);
                        }
                    }

                    obj.FetchComplete = true;
                }
                catch (Exception)
                {
                    MessageBox.Show("There was an error fetching the data. The application will now close.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    Application.Exit();
                }
            }
        }

        /// <summary>
        /// Represents a container for easy access to a solar system's attributes.
        /// </summary>
        public class EveLocation
        {
            public SolarSystem SolarSystem
            {
                get;
                set;
            }

            public Region Region
            {
                get;
                set;
            }

            /// <summary>
            /// Returns an EveLocation object filled with information about a solar system.
            /// </summary>
            /// <param name="ssid">The CCP DB id indicating the solar system.</param>
            /// <returns>An EveLocation object filled with information about the supplied solar system.</returns>
            public static EveLocation GetLocation(string ssid)
            {
                if (!EveData.Data.SolarSystems.ContainsKey(ssid))
                {
                    return null;
                }
                return EveLocation.GetLocation(EveData.Data.SolarSystems[ssid]);
            }

            /// <summary>
            /// Returns an EveLocation object filled with information about a solar system.
            /// </summary>
            /// <param name="ss">The Solar System object to be used.</param>
            /// <returns>An EveLocation object filled with information about the supplied solar system.</returns>
            public static EveLocation GetLocation(SolarSystem ss)
            {
                if (ss == null)
                {
                    return null;
                }

                if (!EveData.Data.Regions.ContainsKey(ss.RegionID))
                {
                    return null;
                }

                return new EveLocation(ss, EveData.Data.Regions[ss.RegionID]);
            }

            public EveLocation()
                : this(null, null)
            {
            }

            public EveLocation(SolarSystem ss, Region r)
            {
                this.SolarSystem = ss;
                this.Region = r;
            }
        }

        /// <summary>
        /// An object that contains information about a solar system.
        /// </summary>
        public class SolarSystem
        {
            public string ID
            {
                get;
                set;
            }

            public string Name
            {
                get;
                set;
            }

            public double Security
            {
                get;
                set;
            }

            public string RegionID
            {
                get;
                set;
            }

            public SolarSystem(Hashtable h)
            {
                this.ID = ((double)h["solarSystemID"]).ToString();
                this.Name = (string)h["solarSystemName"];
                this.Security = (double)h["security"];
                this.RegionID = ((double)h["regionID"]).ToString();
            }

            public SolarSystem(string sysID, string sysName, double sysSec, string sysRegion)
            {
                this.ID = sysID;
                this.Name = sysName;
                this.Security = sysSec;
                this.RegionID = sysRegion;
            }
        }

        /// <summary>
        /// An object that contains information about a region.
        /// </summary>
        public class Region
        {
            public string ID
            {
                get;
                set;
            }

            public string Name
            {
                get;
                set;
            }

            public Region(Hashtable h)
            {
                this.ID = ((double)h["regionID"]).ToString();
                this.Name = (string)h["regionName"];
            }
        }

        /// <summary>
        /// An object that contains information about a type in Eve.
        /// </summary>
        public class Type
        {
            public string ID
            {
                get;
                set;
            }

            public string Name
            {
                get;
                set;
            }

            /// <summary>
            /// A list of required skills to use this type. (if any)
            /// </summary>
            public List<EveData.SkillRequirement> SkillRequirements
            {
                get;
                set;
            }

            public Type(Hashtable h)
            {
                this.ID = ((double)h["typeID"]).ToString();
                this.Name = (string)h["typeName"];
                this.SkillRequirements = new List<SkillRequirement>();
            }
        }

        /// <summary>
        /// An object that contains information about a station.
        /// </summary>
        public class Station
        {
            public string ID
            {
                get;
                set;
            }

            public string Name
            {
                get;
                set;
            }

            public string SolarSystemID
            {
                get;
                set;
            }

            public string RegionID
            {
                get;
                set;
            }

            public Station(Hashtable h)
            {
                this.ID = ((double)h["stationID"]).ToString();
                this.Name = (string)h["stationName"];
                this.SolarSystemID = ((double)h["solarSystemID"]).ToString();
                this.RegionID = ((double)h["regionID"]).ToString();
            }
        }

        /// <summary>
        /// An object that contains information about an agent.
        /// </summary>
        public class Agent
        {
            public string ID
            {
                get;
                set;
            }

            public Agent(Hashtable h)
            {
                this.ID = ((double)h["agentID"]).ToString();
            }
        }

        /// <summary>
        /// Container class to represent skill requirements.
        /// </summary>
        public class SkillRequirement
        {
            /**
                 * - dgmAttributeTypes contains the types of attributes (requiredSkill1, requiredSkill2)
                 *      attributeID 182 = requiredSkill1
                 *      attributeID 183 = requiredSkill2
                 *      attributeID 184 = requiredSkill3
                 *      
                 *      attributeID 277 = requiredSkill1Level
                 *      attributeID 278 = requiredSkill2Level
                 *      attributeID 279 = requiredSkill3Level
                 **/

            public enum RequirementOrder
            {
                First = 0,
                Second = 1,
                Third = 2
            }

            /// <summary>
            /// Contains the attribute ID's used in the CCP database for the first, second and third skill.
            /// </summary>
            public static int[] AttribIDByOrder = { 182, 183, 184 };
            /// <summary>
            /// Contains the level ID's used in the CCP database for the first, second and third skill.
            /// </summary>
            public static int[] LevelIDByOrder = { 277, 278, 279 };

            /// <summary>
            /// Indicates the order in which Eve requires the skills. (not that that actually makes a difference)
            /// </summary>
            public RequirementOrder Order
            {
                get;
                set;
            }

            /// <summary>
            /// Pointer to the Type object containing the skill.
            /// </summary>
            public Type Skill
            {
                get;
                set;
            }

            public int Level
            {
                get;
                set;
            }

            public SkillRequirement(Type s, int lev, RequirementOrder o)
            {
                this.Skill = s;
                this.Level = lev;
                this.Order = o;
            }
        }
    }
}
