﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.IO;

namespace EveDataEncoder
{
    class Program
    {
        static SqlConnection conn;

        static void Main(string[] args)
        {
            string dbserver = "AD\\SQLEXPRESS";
            string dbname = "ebs_DATADUMP";

            // tables/columns to fetch, "tablename" => { "field1", "field2" }
            Dictionary<string, string[]> tables = new Dictionary<string, string[]>();
            tables.Add("invTypes", new string[] { "typeID", "typeName" });
            tables.Add("mapSolarSystems", new string[] { "regionID", "solarSystemID", "solarSystemName", "security" });
            tables.Add("mapRegions", new string[] { "regionID", "regionName" });
            tables.Add("staStations", new string[] { "stationID", "solarSystemID", "regionID", "stationName" });
            tables.Add("agtAgents", new string[] { "agentID" });

            //////////////////////////////////////////////////////////////////////////////////////////
            // Don't modify below this line if you don't know what you're doing.                    //
            //////////////////////////////////////////////////////////////////////////////////////////

            bool fetchskills = false;
            bool fetchdata = false;

            foreach (string arg in args)
            {
                if (arg == "--help" || arg == "-h" || arg == "-?" || arg == "/?")
                {
                    Console.WriteLine("Correct usage: EveDataEncoder [options]");
                    Console.WriteLine();
                    Console.WriteLine("Available options:");
                    Console.WriteLine();
                    Console.WriteLine("\t--help | -h | -?");
                    Console.WriteLine("\tDisplays this list.");
                    Console.WriteLine();
                    Console.WriteLine("\t--skills | -s");
                    Console.WriteLine("\tFetches the required skills for items from the database.");
                    Console.WriteLine();
                    Console.WriteLine("\t--data | -d");
                    Console.WriteLine("\tFetches the static data from the database. (agents, station, types, etc.)");
                    return;
                }
                else if (arg == "--skills" || arg == "-s")
                {
                    fetchskills = true;
                }
                else if (arg == "--data" || arg == "-d")
                {
                    fetchdata = true;
                }
            }

            Console.WriteLine("Fetching data from MSSQL...");
            Console.WriteLine();
            Console.WriteLine();

            Program.conn = new SqlConnection("Data Source=" + dbserver + ";" + 
                                             "Trusted_Connection=yes;" +
                                             "Database=" + dbname + ";" + 
                                             "Connection Timeout=30");
            try
            {
                Program.conn.Open();
            }
            catch (Exception e)
            {
                Console.WriteLine("Exception thrown! (conn.open) " + e.ToString());
                return;
            }

            if (fetchdata)
            {
                try
                {
                    foreach (KeyValuePair<string, string[]> pair in tables)
                    {
                        Console.WriteLine("From table [{0}], fetching colums: {1}", pair.Key, String.Join(", ", pair.Value));
                        Console.WriteLine("Writing to file {0}.js...", pair.Key);
                        Console.WriteLine();

                        // new table, fetch data, then create file
                        ArrayList jsondata = new ArrayList();

                        SqlCommand cmd = new SqlCommand(String.Format("SELECT {0} FROM {1}", String.Join(",", pair.Value), pair.Key), Program.conn);
                        SqlDataReader reader = cmd.ExecuteReader();

                        while (reader.Read())
                        {
                            // new row
                            Hashtable table = new Hashtable();

                            foreach (string column in pair.Value)
                            {
                                if (reader[column] is string)
                                {
                                    // clean up the string, if needs be
                                    string str = (reader[column] as string);
                                    str = str.Replace("\"", "\\\""); // replace " with \"
                                    table[column] = str;
                                }
                                else
                                {
                                    table[column] = reader[column];
                                }
                            }

                            jsondata.Add(table);
                        }

                        string json = APIcheck.JSON.JsonEncode(jsondata);
                        File.WriteAllText(pair.Key + ".js", json);

                        reader.Close();
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine("Exception thrown! (reader) " + e.ToString());
                    return;
                }
            }

            if (fetchskills)
            {
                /**
                 * This one is a bit harder.
                 * The data is split across 3 tables;
                 * - dgmAttributeTypes contains the types of attributes (requiredSkill1, requiredSkill2)
                 *      attributeID 182 = requiredSkill1
                 *      attributeID 183 = requiredSkill2
                 *      attributeID 184 = requiredSkill3
                 *      
                 *      attributeID 277 = requiredSkill1Level
                 *      attributeID 278 = requiredSkill2Level
                 *      attributeID 279 = requiredSkill3Level
                 * - dgmTypeAttributes contains the values for said attributes, in either valueInt or valueFloat
                 * - invTypes contains the item types and their names
                 * 
                 * What we wanna do here is loop through invTypes, query they attributes for every item, and save them if they have required skills.
                 **/
                
                try
                {
                    Console.WriteLine("Fetching the items and their required skills.");
                    Console.WriteLine();

                    SqlCommand cmd = new SqlCommand(EveDataEncoder.Properties.Resources.Skillquery, Program.conn);
                    SqlDataReader reader = cmd.ExecuteReader();

                    Hashtable jsondata = new Hashtable();
                    while (reader.Read())
                    {
                        // a table of tables
                        // (id: (attribID: x, attribValue: x))
                        if (!jsondata.ContainsKey(reader["invTypeID"]))
                        {
                            jsondata.Add(reader["invTypeID"], new Hashtable());
                        }

                        Hashtable itemtable = (Hashtable)jsondata[reader["invTypeID"]];
                        itemtable.Add(reader["attribID"], reader["attribValue"]);
                    }
                    reader.Close();

                    string json = APIcheck.JSON.JsonEncode(jsondata);
                    File.WriteAllText("skillrequirements.js", json);
                }
                catch (Exception e)
                {
                    Console.WriteLine("Exception thrown! (reader) " + e.ToString());
                    return;
                }
            }

            Console.WriteLine("Application complete.");

            Program.conn.Close();
        }
    }
}
